"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SocketIORouter = exports.SocketIOPlugin = void 0;
const debug = require('debug')('methodus');
const http = require("http");
const metadataKey = 'methodus';
const socketIO = require("socket.io");
const server_1 = require("@methodus/server");
const commons_1 = require("@methodus/server/commons");
class SocketIOPlugin extends commons_1.BaseServer {
    constructor(options, httpServer) {
        super();
        this.classRouters = [];
        if (httpServer) {
            this._app = socketIO(httpServer);
        }
        else {
            this._app = socketIO.listen(options.port, { transports: ['polling', 'websocket'] });
        }
        global.socketioEngine = this._app;
        let nsp = this._app;
        if (options && options.nsp) {
            nsp = this._app.of(options.nsp);
        }
        nsp.on('connection', (socket) => {
            this.socketHandler(socket);
        });
        nsp.use((socket, next) => {
            next();
        });
    }
    close() {
        this._app.close();
    }
    useClass(classType) {
        this.classRouters.push(classType);
    }
    socketHandler(socket) {
        if (!socket.attached) {
            this.classRouters.forEach((item) => {
                return new SocketIORouter(item, socket);
            });
            socket.attached = true;
        }
    }
    static register(server, parentServer) {
        console.log(`> Starting SOCKETIO server`);
        const httpServer = server_1.Servers.get(server.instanceId, 'http') || http.createServer(parentServer._app);
        server_1.Servers.set(server.instanceId, 'http', httpServer);
        const app = new SocketIOPlugin(server.options, httpServer);
        server_1.Servers.set(server.instanceId, server.type.name, app);
    }
}
exports.SocketIOPlugin = SocketIOPlugin;
class SocketIORouter {
    constructor(obj, socket) {
        this.prefix = '';
        this.router = null;
        const proto = commons_1.default.util.maybeProto(obj);
        const methodus = commons_1.default.util.maybeMethodus(obj)[obj.name];
        const existingClassMetadata = Reflect.getOwnMetadata(metadataKey, proto) || {};
        existingClassMetadata.returnMessages = true;
        Reflect.defineMetadata(metadataKey, existingClassMetadata, proto);
        Object.keys(methodus._descriptors).forEach((itemKey) => {
            const item = methodus._descriptors[itemKey];
            debug('activating controller method', item, methodus);
            socket.on(item.verb + '_' + item.route, (data, callback) => __awaiter(this, void 0, void 0, function* () {
                debug('activating controller method', itemKey, data);
                const paramsMap = Reflect.getOwnMetadata('params', proto, itemKey) || [];
                debug('method params', itemKey, paramsMap);
                const functionArgs = [];
                methodus._descriptors[itemKey].params.forEach((xitem) => {
                    functionArgs[xitem.index] = data[xitem.name];
                });
                try {
                    const result = yield proto[itemKey](...functionArgs, socket, data);
                    debug('result is:', result);
                    callback(result);
                }
                catch (error) {
                    callback(error);
                }
            }));
        });
    }
}
exports.SocketIORouter = SocketIORouter;
//# sourceMappingURL=socketio.js.map